#!/opt/puppet/bin/ruby

require 'trollop'
require 'json'
require 'winrm'

@opts = Trollop::options do
  opt :server, 'scvmm server address', :type => :string, :required => true
  opt :port, 'scvmm server port', :default => 5986
  opt :username, 'scvmm server username', :type => :string, :required => true
  opt :password, 'scvmm server password', :type => :string, :default => ENV['PASSWORD']
  opt :quick, 'do quick check if device is scvmm', :type => :boolean
  opt :output, 'output facts to a file', :type => :string, :required => false
  opt :credential_id, 'dummy value for ASM, not used'
end

def winrm
  endpoint = "https://#{@opts[:server]}:#{@opts[:port]}/wsman"
  WinRM::WinRMWebService.new(
      endpoint, :ssl,
      :user => @opts[:username],
      :pass => @opts[:password],
      :disable_sspi => true,
      :no_ssl_peer_verification => true
  )
end

def facts(options = {})
  import = 'Import-Module VirtualMachineManager;'

  data = Hash.new

  {
    :hostgroup => import + 'Get-SCVMHostGroup -VMMServer localhost | Select Path | ConvertTo-Json',
    :host => import + 'Get-SCVMHost -VMMServer localhost | Select Name,{$_.HostCluster.ClusterName},{$_.VMHostGroup.Path} | ConvertTo-Json',
    :vm => import + 'Get-SCVirtualMachine -VMMServer localhost | Select Name,MarkedAsTemplate,HostGroupPath,{$_.OperatingSystem.Name},HostType | ConvertTo-Json',
    :template => import + 'Get-Template -VMMServer localhost | Select Name,{$_.OperatingSystem.Name} | ConvertTo-Json',
  }.each do |res_type, command|
    result = ''
    winrm.create_executor.run_powershell_script(command) do |stdout, stderr|
      result += stdout.strip unless stdout.nil?
    end
    begin
      data[res_type] = JSON.parse(result)
    rescue JSON::ParserError
      data[res_type] = []
    end
    data[res_type] = [data[res_type]] unless data[res_type].is_a? Array

    break if options[:quick]
  end
  data
end

if @opts[:password].nil?
  puts "Error: option --password must be specified."
  puts "Try --help for help."
  exit 1
elsif !@opts[:quick] && @opts[:output].nil?
  puts 'Must include fact output argument (--output).'
  puts 'Type --help for help.'
  exit 1
end

# NOTE: Structured facts currently cause UI to break, and the UI
# relies on all scvmm fact being a JSON-encoded string in the "value"
# facts. UI also expects "name" to be of form TYPE-IP.
#
# Also populating vendor and serial_number very naively here so that
# they don't have to be hard-coded in the UI
name = "scvmm-#{@opts[:server]}"
out = {
  :name => name,
  :vendor => "Microsoft",
  :serial_number => name,
  :value => facts(@opts).to_json
}.to_json

if @opts[:quick]
  puts out
else
  File.write(@opts[:output], out)
end