unless Kernel.respond_to?(:require_relative)
  module Kernel
    def require_relative(path)
      require File.join(File.dirname(caller[0]), path.to_str)
    end
  end
end

require_relative '../../provider/winrm'

Puppet::Type.type(:iscsi_target).provide(:default, :parent => Puppet::Provider::Winrm) do

  def exists?
    false
  end

  def disconnect
    @nodeaddress.each do |address|
      command = ps_cmd('Disconnect-IscsiTarget',
        {
          :NodeAddress => quote(address),
          :Confirm => '$false',
        }
      )
      winrm_ps(command)
    end
  end

  def exist_cmd
    command = ps_cmd( 'Get-IscsiTarget', {}, :terminate => false)
  end

  def iscsi_targets
    if resource[:storage_match_pattern].empty?
      '$targets = Get-IscsiTarget;'
    else
      "$targets = Get-IscsiTarget | ?{$_.NodeAddress -match #{quote(resource[:storage_match_pattern])}};"
    end

  end

  def get_ports
    '$ports = Get-InitiatorPort;'
  end

  def connect
    ret_val = true
    target_portal = "$target_portal = @('#{resource[:target_portal_address]}' -split ',');"
    loop_command = "foreach ($target in $targets) { "
    loop_command += " foreach ($port in $ports) { "
    loop_command += " #{target_command} "
    loop_command += "} }"
    command = target_portal + iscsi_targets + get_ports + loop_command
    begin
      node_vmm(command)
    rescue => ex
      Puppet.debug("Response: #{ex.inspect}")
      ex.message.match(/Target Already Exists/) ? ret_val = true : ret_val = false
    end
    ret_val
  end

  def target_command
    ps_cmd( 'Connect-IscsiTarget',
      {
          :NodeAddress => "$target.NodeAddress",
          :IsPersistent => resource[:is_persistent],
          :IsMultipathEnabled => resource[:is_multipath_enabled],
          :InitiatorInstanceName => "$port.InstanceName",
      }
    )
  end

end
