unless Kernel.respond_to?(:require_relative)
  module Kernel
    def require_relative(path)
      require File.join(File.dirname(caller[0]), path.to_str)
    end
  end
end

require_relative '../../provider/winrm'

Puppet::Type.type(:iscsi_target_portal).provide(:default, :parent => Puppet::Provider::Winrm) do

  def exists?
    # Do not truncate output:
    command  = exist_cmd + '| Out-String -width 4096'
    result = winrm_ps(command)
    result.include? resource[:name]
  rescue RuntimeError
    false
  end

  def create
    target_portal = "$targetportal = @('#{resource['target_portal_address']}' -Split ',')  ;"
    loop_command = "foreach ($portal in $targetportal) { "
    loop_command += "new-iscsitargetportal -targetportaladdress $portal "
    loop_command += "}"
    command = target_portal + loop_command
    begin
      node_vmm( command )
    rescue => ex
      Puppet.debug("Error encountered while configuring iscsi target portal configuration: #{ex.message}")
      Puppet.debug('In case of multiple iscsi networks errors in powershell command is expected, Ignoring errors')
    end

  end

  def destroy
    command = ps_cmd('Remove-IscsiTargetPortal',
      {
        :InitiatorInstanceName => quote(resource[:name]),
        :TargetPortalAddress => resource[:target_portal_address],
        :Confirm => '$false',
      }
    )

    winrm_ps(command)
  end

  def exist_cmd
    command = ps_cmd( 'Get-IscsiTargetPortal',
      :terminate => false
    )
  end

  def get_ports
    '$ports = Get-InitiatorPort;'
  end

  def create_cmd
    ps_cmd( 'New-IscsiTargetPortal',
      {
        :InitiatorInstanceName => quote(resource[:name]),
        :TargetPortalAddress => resource[:target_portal_address],
        :TargetPortalPortNumber => resource[:target_portal_port_number],
        :AuthenticationType => resource[:authentication_type],
        :ChapUsername => resource[:chap_username],
        :ChapSecret => resource[:chap_secret],
      }
    )
  end
end
