unless Kernel.respond_to?(:require_relative)
  module Kernel
    def require_relative(path)
      require File.join(File.dirname(caller[0]), path.to_str)
    end
  end
end

require_relative '../../provider/winrm'

Puppet::Type.type(:sc_virtual_machine).provide(:default, :parent => Puppet::Provider::Winrm) do

  def exists?
    filter = '| Select Name'
    result = vmm(exist_cmd+filter)
    result and result.include?(resource[:name])
  end

  def create
    if resource[:template]
      get_template = "$Template = Get-SCVMTemplate -VMMServer localhost | where { $_.Name -eq #{quote(resource[:template])} };"
      template_disk_size = vmm("$Template = Get-SCVMTemplate -VMMServer localhost | where { $_.Name -eq #{quote(resource[:template])}} ; echo $Template.TotalVHDCapacity")
      template_disk_size = Integer(template_disk_size) / (1024 * 1024 * 1024)
      if resource[:vm_host].nil?
        get_vmhost = "$HostRating = Get-SCVMHostRating -VMTemplate $Template -DiskSpaceGB #{template_disk_size} -VMHost $VMhosts -VMName #{quote(resource[:name])} -ReturnFirstSuitableHost:$true ; $VMHost = Get-SCVMHost -computername $HostRating.name ;"
      else
        get_vmhost = "$VMHost = Get-SCVMHost -computername #{resource[:vm_host]};"
      end

      command = get_template + cluster + cluster_host_cmd +  get_vmhost + template_cmd
      Puppet.debug("Create Command: #{command}")
      vmm(command)
    else
      Puppet.error "Not a supported VM deployment configuration."
    end
  end

  def destroy
    command = "#{exist_cmd} | Remove-SCVirtualMachine"
    vmm(command)
  end

  def cluster_cmd
    ps_cmd('Get-SCVMHostCluster',
       {
         :VMMServer => 'localhost',
       },
       :terminate => false,
    )

  end

  def cluster
    "$cluster = Get-SCVMHostCluster -name #{quote(resource[:vm_cluster])}; "
  end

  def cluster_host_cmd
    "$VMHosts = Get-SCVMHost -VMHostCluster $cluster ; "
  end


  def exist_cmd
    ps_cmd( 'Get-SCVirtualMachine',
      {
        :VMMServer => 'localhost',
        :Name => quote(resource[:name]),
      },
      :terminate => false,
    )
  end

  def get_username
    resource[:domain_username]
  end

  def get_password
    resource[:domain_password]
  end

  def template_cmd
    cmd = ps_cmd( 'New-SCVirtualMachine',
      {
        :Name => quote(resource[:name]),
        :Description => quote(resource[:description]),
        :VMTemplate => '$Template',
        :VMHost => '$VMHost',
        :Path => quote(resource[:path]),
        :StartAction => resource[:start_action],
        :StopAction => resource[:stop_action],
        :CPUCount => resource[:cpu_count],
        :MemoryMB => resource[:memory_mb],
        :RunAsynchronously => resource[:async],
        :Domain => resource[:domain],
        :DomainJoinCredential => resource[:domain_username] ? '$credential' : nil,
        :HighlyAvailable => resource[:highly_available],
        :ProductKey => resource[:product_key],
      },
      :terminate => false,
    )

    if resource[:domain_username]
      credential + cmd
    else
      cmd
    end
  end
end
