unless Kernel.respond_to?(:require_relative)
  module Kernel
    def require_relative(path)
      require File.join(File.dirname(caller[0]), path.to_str)
    end
  end
end

require_relative '../../provider/winrm'

Puppet::Type.type(:sc_virtual_network_adapter).provide(:default, :parent => Puppet::Provider::Winrm) do

  def exists?
    filter = ''
    filter += "| Where { $_.SlotID -eq #{resource[:slot_id]} }" if resource[:slot_id]
    filter += '| Select SlotID, MacAddressType, MacAddress, IPv4AddressType, IPv6AddressType, VLANEnabled, VLANID, LogicalNetwork, VirtualNetwork | ConvertTo-JSON'
   
    existence_command = (vm_cmd+exist_cmd+filter)
    Puppet.debug ("Exists command: #{existence_command}") 
    result = vmm(existence_command)
    Puppet.debug("Result: #{result}")
    begin
      @nic = PSON.parse(result)
    rescue PSON::ParserError
      @nic = nil
    end
    Puppet.debug("Existence response: #{@nic}")
    @nic = @nic[0] if !@nic.nil? and @nic.is_a?(Array)
    @nic and @nic.include? 'SlotId'
  end

  def create
    @create = true
    creation_command = (vm_cmd + power_off_vm + create_cmd + power_on_vm )
    Puppet.debug ("Creation command: #{creation_command}")
    vmm(creation_command)
  end

  def destroy
    command = "#{exist_cmd} | Remove-SCVirtualMachine"
    vmm(command)
  end

  def flush
    vmm(power_off_vm + set_cmd + power_on_vm) if ! @create and @flush
  end

  def nic_type
    {
      0 => :dynamic,
      1 => :static
    }
  end
  
  def ipv4_address_type
    value = @nic['IPv4AddressType']
    nic_type[value]
  end

  def ipv4_address_type=(value)
    @flush = true
  end

  def ipv6_address_type
    value = @nic['IPv6AddressType']
    nic_type[value]
  end

  def ipv6_address_type=(value)
    @flush = true
  end

  def mac_address_type 
    value = @nic['MACAddressType']
    nic_type[value]
  end

  def mac_address_type=(value)
    if nic_type[@nic['MACAddressType']] == :dynamic &&
        @nic['MACAddress'] != '00-00-00-00-00-00'
      @flush = false
    else
      @flush = true
    end
  end

  def mac_address
    case resource[:mac_address_type]
    when :static
      if resource[:mac_address] == '00-00-00-00-00-00'
        '00-00-00-00-00-00'
      else
        @nic['MACAddress'].gsub(':', '-')
      end
    when :dynamic
      resource[:mac_address]
    end
  end

  def mac_address=(value)
    @flush = true
  end

  def vlan_enabled
    @nic['VLanEnabled']
  end

  def vlan_id
    case resource[:vlan_enabled]
      when '$true'
        @nic['VLanID']
      else
        resource[:vlan_id]
    end
  end

  def vlan_id=(value)
    @flush = true
  end

  def virtual_network
    @nic['VirtualNetwork']
  end

  def virtual_network=(value)
    @flush = true
  end

  def logical_network
    (@nic['LogicalNetwork'] || {})['Name']
  end

  def logical_network=(value)
    @flush = true
  end

  def vm_cmd
    ps_cmd( 'Get-SCVirtualMachine',
      {
        :VMMServer => 'localhost',
        :Name => quote(resource[:vm]),
      },
      :variable => '$VM',
    )
  end

  def exist_cmd
    ps_cmd( 'Get-SCVirtualNetworkAdapter',
      {
        :VM => '$VM',
      },
      :terminate => false,
    )
  end

  def mac_address_value
    if resource[:mac_address_type] == :static and resource[:mac_address]
      mac_address = quote(resource[:mac_address])
    else
      mac_address = nil
    end
  end

  def set_cmd
    adapter_cmd = '$Adapters = Get-SCVirtualNetworkAdapter -VM $VM;'
    lnet_cmd = "$LogicalNet = Get-SCLogicalNetwork -Name #{quote(resource[:logical_network])};"
    cmd = ps_cmd( 'Set-SCVirtualNetworkAdapter',
      {
        :VirtualNetworkAdapter => "$Adapters[#{resource[:slot_id]}]",
        :EnableMACAddressSpoofing => resource[:enable_mac_address_spoofing],
        :EnableVMNetworkOptimization => resource[:enable_vm_network_optimization],
        :IPv4AddressType => resource[:ipv4_address_type],
        :IPv6AddressType => resource[:ipv6_address_type],
        :LogicalNetwork => $LogicalNet,
        :VirtualNetwork => resource[:virtual_network] ? '$VritualNet': nil,
        :MACAddress => mac_address_value,
        :MACAddressType => resource[:mac_address_type],
        :VLANEnabled => resource[:vlan_enabled],
        :VLANId => resource[:vlan_id],
        :RunAsynchronuosly => resource[:async],
      },
      :terminate => false,
    )
    Puppet.debug("cmd: #{cmd}")

    cmd = vm_cmd + adapter_cmd + cmd
  end

  def power_on_vm
    command = ";Import-Module VirtualMachineManager; Get-VMMServer -ComputerName localhost; Start-SCVirtualMachine -VM #{quote(resource[:vm])};"
  end

  def power_off_vm
    command = "Import-Module VirtualMachineManager; Get-VMMServer -ComputerName localhost; Stop-SCVirtualMachine -VM #{quote(resource[:vm])};"
  end

  def create_cmd 
    lnet_cmd = "$LogicalNet = Get-SCLogicalNetwork -Name #{quote(resource[:logical_network])};"
    vnet_cmd = "$VirtualNet = Get-SCVirtualNetwork -Name #{quote(resource[:virtual_network])};"
    cmd = ps_cmd( 'New-SCVirtualNetworkAdapter',
      {
        :VM => '$VM',
        :EnableMACAddressSpoofing => resource[:enable_mac_address_spoofing],
        :EnableVMNetworkOptimization => resource[:enable_vm_network_optimization],
        :IPv4AddressType => resource[:ipv4_address_type],
        :IPv6AddressType => resource[:ipv6_address_type],
        :LogicalNetwork => resource[:logical_network] ? '$LogicalNet': nil,
        :VirtualNetwork => resource[:virtual_network] ? '$VritualNet': nil,
        :VLANEnabled => resource[:vlan_enabled],
        :VLANId => resource[:vlan_id],
        :RunAsynchronuosly => resource[:async],
      },
      :terminate => false,
    )

    cmd = lnet_cmd + cmd if resource[:logical_network]
    cmd = vnet_cmd + cmd if resource[:virtual_network]
    cmd += " -Synthetic"

    cmd
  end

  def get_username
    resource[:domain_username]
  end

  def get_password
    resource[:domain_password]
  end
end
