unless Kernel.respond_to?(:require_relative)
  module Kernel
    def require_relative(path)
      require File.join(File.dirname(caller[0]), path.to_str)
    end
  end
end

require_relative '../../provider/winrm'

vmware_module = Puppet::Module.find('vmware_lib', Puppet[:environment].to_s)
require File.join vmware_module.path, 'lib/puppet_x/vmware/util'

Puppet::Type.type(:scvm_cluster_storage).provide(:default, :parent => Puppet::Provider::Winrm) do

  def exists?
    # TODO: This is not usable as a managed resource:
    false
  end

  def create
    vmm(refresh_cluster_cmd)
    vmm(create_cmd_csv)
    out = vmm(create_csv)
    # Check the error in the response
    Puppet.debug("Output of the storage configuration: #{out}")
    raise(Exception,'Failed to mount the storage') if out.match(/VMM is unable to complete the request/)
    validate_cluster_storage
  end

  def destroy
    Puppet.error('Does not support removal of cluster storage.')
  end

  def create_cmd_csv
    prefix = ps_cmd('Get-SCVMHostGroup',
      {
        :VMMServer => 'localhost',
      },
      :terminate => false,
      :variable => '$hostGroup'
    ) + "| where { $_.path -eq #{quote(resource[:path])} });"

  end

  def refresh_cluster_cmd()
    # Refresh host cluster
    cluster_name = "#{resource[:name]}" + "." + "#{resource[:fqdn]}"
    Puppet.debug "Refreshing host cluster"
    cluster = "$cluster = (Get-SCVMHostCluster -name #{quote(cluster_name)});"
    import_details = "$importdetails = Get-VMMServer -ComputerName localhost;"
    command = ps_cmd('Refresh-VMHostCluster',
      {
        :VMHostCluster => '$cluster',
       }
     )
    command = credential + import_details + cluster + command
  end

  def validate_cluster_storage
    retry_counter = 0
    begin
      vmm(refresh_cluster_cmd)
      Puppet.debug("Wait for a minute before validating cluster shared volues")
      sleep 60
      shared_cluster_volumes = (vmm(cluster_info_cmd) || '')
      Puppet.debug("Shared volumes from vmm: #{shared_cluster_volumes}")
      cluster_shared_vol = ( shared_cluster_volumes.scan(/SharedVolumes\s+:\s*\{(.*?)\}/m).flatten.first || '' )
      Puppet.debug("Cluster Shared Volume: #{cluster_shared_vol}")
      raise('No cluster storage created') if cluster_shared_vol.size == 0
    rescue
      if retry_counter <= 5
        Puppet.debug("Not able to find CSV for attempt : #{retry_counter + 1}")
        retry_counter += 1
        sleep 120
        retry
      else
        raise('No cluster storage created')
      end
    end
  end

  def cluster_info_cmd
    cluster_name = "#{resource[:name]}" + "." + "#{resource[:fqdn]}"
    command = "Get-SCVMHostCluster -vmmserver localhost -Name  #{quote(cluster_name)}"
  end

  def create_csv

    cluster_name = "#{resource[:name]}" + "." + "#{resource[:fqdn]}"
    import_details = "$importdetails = Get-VMMServer -ComputerName localhost;"
    temp_command = "$SMLunIdArray = @();"
    temp_command.concat("$Cluster = (Get-SCVMHostCluster -Name  #{quote(cluster_name)});")
    temp_command.concat("$VMHost = (Get-SCVMHost | where name -eq #{quote(resource[:host])});")
    temp_command.concat("$hostDisk = (Get-SCStorageDisk -VMHost $VMHost | where IsClustered -eq $False | where Classification -like \"Remote Storage\" | select SMLunId);")
    temp_command.concat("$hostDiskList = $hostDisk.SMLunId ; ")
    # Converting Host Volume to Available Storage at Cluster
    temp_command.concat("foreach ($SMLunId in $hostDiskList) {
                         $hostDisk = Get-SCStorageDisk -VMHost $VMHost | where { $_.SMLunId -eq $SMLunId }
                         Mount-SCStorageDisk -StorageDisk $hostDisk -MasterBootRecord -FullFormat
                         Set-SCVMHostCluster -VMHostCluster $Cluster -ClusterReserve \"0\" };")

    # Converting Available Volume to CSV 
    temp_command.concat("$clusterVolumes = Get-SCVMHostCluster -Name #{quote(cluster_name)} | select AvailableVolumes;")
    temp_command.concat("$clusterVolumesList = $clusterVolumes.AvailableVolumes | where Capacity -gt \"2000000000\" | select HostDiskID; ")
    temp_command.concat("$DiskIdList = $clusterVolumesList.HostDiskID.Guid; ")
    temp_command.concat("foreach ($clusterVolume in $DiskIdList) {
                         $clusterVolumes = Get-SCStorageDisk | where { $_.DiskId -eq $clusterVolume }
                         Mount-SCStorageDisk -StorageDisk $clusterVolumes -CreateClusterSharedVolume
                         Set-SCVMHostCluster -VMHostCluster $Cluster -ClusterReserve \"0\" \};")


    # setting quorum Disk
    quorum_vol = ""
    quorum_vol.concat("$clusterqourm = get-clusterquorum -cluster #{quote(resource[:name])} | select $_.QuorumResource;")
    quorum_vol.concat("$clusterqourmstate = $clusterqourm.QuorumResource.state;")
    quorum_vol.concat("if ([string]::IsNullOrEmpty($clusterqourmstate)) {")
    quorum_vol.concat("$StorageNames = Get-ClusterResource -Cluster #{quote(resource[:name])} | where OwnerGroup -eq \"Available Storage\" |select name; ")
    quorum_vol.concat("Set-ClusterQuorum -NodeAndDiskMajority $StorageNames[0].name -Cluster #{quote(cluster_name)}")
    quorum_vol.concat("};")
    quorum_vol.concat("$cluster = Get-SCVMHostCluster -name #{quote(cluster_name)};")
    quorum_vol.concat("Refresh-VMHostCluster -VMHostCluster $cluster;")

    command = credential + import_details + temp_command + quorum_vol

  end

end
