unless Kernel.respond_to?(:require_relative)
  module Kernel
    def require_relative(path)
      require File.join(File.dirname(caller[0]), path.to_str)
    end
  end
end

require_relative '../../provider/winrm'

Puppet::Type.type(:scvm_host).provide(:default, :parent => Puppet::Provider::Winrm) do

  def exists?
    result = vmm("Get-SCVMHost -ComputerName #{resource[:name]} -VMMServer localhost | Select Name ")
    result and result.downcase.include?(resource[:name].downcase)
  rescue RuntimeError
    false
  end

  def create
    command = credential + "Add-SCVMHost -ComputerName \"#{resource[:name]}\" -VMMServer localhost -Credential $credential"
    if resource[:path] 
      prefix = "$hostGroup = (Get-SCVMHostGroup -VMMServer localhost | where {$_.path -eq \"#{resource[:path]}\"});"
      command = prefix + command + " -VMHostGroup $hostGroup"
    end
    command += ' -RunAsynchronously' if resource[:async] == :true
    vmm(command)
    # Some property can only be set afterwards:
    self.send(:management_account=, resource[:management_account]) if resource[:management_account]
  end

  def destroy
    command = credential + "Get-SCVMHost -ComputerName #{resource[:name]} -VMMServer localhost | Remove-SCVMHost -Credential $credential"
    command = command + ' -RunAsynchronously' if resource[:async] == :true
    vmm(command)
  end

  def management_account=(value)
    account = "$hostManagementAccount = Get-SCRunAsAccount -VMMServer localhost -Name \"#{resource[:management_account]}\";" 
    command =  credential + account + "Get-SCVMHost -ComputerName \"#{resource[:name]}\" -VMMServer 'localhost' | Set-SCVMHost -Credential $credential -VMHostManagementCredential $hostManagementAccount[0]"
    command += ' -RunAsynchronously' if resource[:async] == :true
    vmm(command)
  end
end
