unless Kernel.respond_to?(:require_relative)
  module Kernel
    def require_relative(path)
      require File.join(File.dirname(caller[0]), path.to_str)
    end
  end
end

require_relative '../../provider/winrm'

vmware_module = Puppet::Module.find('vmware_lib', Puppet[:environment].to_s)
require File.join vmware_module.path, 'lib/puppet_x/vmware/util'

Puppet::Type.type(:scvm_host_adapter).provide(:default, :parent => Puppet::Provider::Winrm) do

  def exists?
    Puppet.debug("Inside exists block, returning false from here")
    false
  end

  def create
    response = node_vmm(create_cmd)
    Puppet.debug("Command Response: #{response}")
  end

  def destroy
    vmm(destroy_cmd)
  end

  def refresh_cluster_cmd()
    # Refresh host cluster
    cluster_name = "#{resource[:name]}" + "." + "#{resource[:fqdn]}"
    Puppet.debug "Refreshing host cluster"
    cluster = "$cluster = (Get-SCVMHostCluster -name #{quote(cluster_name)});"
    import_details = "$importdetails = Get-VMMServer -ComputerName localhost;"
    command = ps_cmd('Refresh-VMHostCluster',
                     {
                         :VMHostCluster => '$cluster',
                     }
    )
    command = credential + import_details + cluster + command
  end

  def exist_cmd
    if resource[:fqdn] and resource[:name] !~ /#{resource[:fqdn]}$/
      cluster_name = resource[:name] + "." + resource[:fqdn]
    else
      cluster_name = resource[:name]
    end

    ps_cmd('Get-SCVMHostCluster',
      {
        :VMMServer => 'localhost',
        :Name => quote(cluster_name),
      },
      :terminate => false
    )
  end

  def create_cmd

    # Live and LM is set for windows 2012 and 2012 r2 version resp.So handled for both.
    cluster_name = "#{resource[:name]}" + "." + "#{resource[:fqdn]}"
    live_command = " $live = Get-ClusterNetworkInterface -cluster #{quote(cluster_name)} | "
    live_command += "Where-Object { $_.name -match #{quote("Live|LM")} };"

    live_command += "$name = Get-ClusterNetwork -Cluster #{quote(resource[:name])} | "
    live_command += "Where-Object {$_.Name -eq $live[0].Network.Name }; "
    live_command += " foreach ( $c in $name ) { $c.role=1 } ; "

    cluster_command = "Get-ClusterResourceType -name #{quote("Virtual Machine")} "
    cluster_command += "-cluster #{quote(resource[:name])} | "
    cluster_command += "Set-ClusterParameter -name MigrationExcludeNetworks "
    cluster_command += "-Value ([String]::Join(\";\",(Get-ClusterNetwork -cluster #{quote(resource[:name])} "
    cluster_command += "| Where-Object { $_.Address -ne $live[0].Network.Address}).ID)) ;"

    network_command = "$network = Get-ClusterNetworkInterface -cluster #{quote(resource[:name])} "
    network_command += "| Where-Object { $_.name -match #{quote("Cluster")} } | select Network; "
    network_command += "$name = Get-ClusterNetwork -Cluster #{quote(resource[:name])} | "
    network_command += "Where-Object {$_.Name -eq $network[0].Network.Name }; "
    network_command += " foreach ( $c in $name ) { $c.role=1 } "
    command = live_command  + cluster_command + network_command
    Puppet.debug("Command: #{command}")
    command

  end

  def destroy_cmd
    Puppet.debug("Not implemented")
  end
end
