require 'pathname'

begin
  require 'puppet_x/puppetlabs/transport'
  require 'puppet_x/puppetlabs/transport/winrm'
rescue LoadError
  require 'pathname' # WORK_AROUND #14073 and #7788
  mod = Puppet::Module.find('transport', Puppet[:environment].to_s)
  require File.join mod.path, 'lib/puppet_x/puppetlabs/transport'
  require File.join mod.path, 'lib/puppet_x/puppetlabs/transport/winrm'
end

class Puppet::Provider::Winrm <  Puppet::Provider
  def vmm(cmd, exitcode = [0])
    load_module = 'Import-Module virtualmachinemanager;'
    command = "#{load_module} #{cmd}"
    winrm_ps(command, exitcode)
  end

  def node_vmm(cmd, exitcode = [0])
    winrm_ps(cmd, exitcode)
  end

  def winrm
    @transport ||= PuppetX::Puppetlabs::Transport.retrieve(
        :resource_ref => resource[:transport],
        :catalog => resource.catalog,
        :provider => "winrm"
    )
    @transport.winrm.set_timeout(3600)
    @transport.winrm
  end

  def winrm_ps(cmd, expected_exitcodes = [0])
    begin
      output = winrm.create_executor.run_powershell_script(cmd)
    rescue => ex
      # WinRM 1.8.1 has a bug where it tries to raise a exception class "WinRMWebServiceError" that does not exist, this works around that bug
      if ex.message.match(/WinRM::HTTP::HttpNegotiate::WinRMWebServiceError|Bad HTTP response returned from server \(400\)/i)
        Puppet.debug("HTTP Negotiate error, retry puppet execution. Error message : %s" % [ex.message])
        sleep 60
        output = winrm.create_executor.run_powershell_script(cmd)
      else
        raise ex
      end
    end

    stdout = output[:data].collect{|line| line[:stdout]}.join
    stderr = output[:data].collect{|line| line[:stderr]}.join
    raise(RuntimeError, stdout+stderr) unless expected_exitcodes.include? output[:exitcode]
    stdout
  end

  def quote(string, option = :double)
    case option
      when :single
        %['#{string}']
      when :double
        %["#{string}"]
      when :double_star
        %["#{string}.*"]
      when :none
        string
      else
        raise ArgumentError, "Invalid quote option: #{option}"
    end
  end

  def ps_cmd(command, arguments = {}, options = {})
    default = {
      :terminate => true,
    }
    options = default.merge(options)

    arguments.each do |arg, value|
      case value
      when TrueClass, FalseClass
        command += " -#{arg}" if value
      when :true, :false
        command += " -#{arg}" if value == :true
      when Array
        command += " -#{arg} #{value.join(', ')}"
      when '$true', '$false' #Powershell true/false
        command += " -#{arg}:#{value}"
      when String, Numeric
        command += " -#{arg} #{value}"
      when Symbol
        command += " -#{arg} #{value.to_s}"
      when nil
        command
      else
        raise ArgumentError, "Unable to process argument #{arg} : #{arg.class} #{value.class} '#{value}'"
      end
    end

    command = "#{options[:variable]} = (#{command})" if options[:variable]
    command += ';' if options[:terminate]
    command += "| #{options[:pipe]}" if options[:pipe]
    command
  end

  # Allows other provider to specify ways to decrypt password.
  def get_username
    resource[:username]
  end

  def get_password
    resource[:password]
  end

  def credential(username = get_username, password = get_password)
    encrypt = ps_cmd(
      "ConvertTo-SecureString #{quote(password, :single)}",
      {
        :AsPlainText => true,
        :Force => true,
      },
      :variable  => '$encrypt',
      :terminate => true,
    )

    cred = ps_cmd('New-Object',
      {
        :TypeName => 'System.Management.Automation.PSCredential',
        :ArgumentList => [ quote(username), '$encrypt' ],
      },
      :variable => '$credential',
    )
    command = encrypt + cred
  end

  def result_map
    Hash.new
  end

  def hashkey_downcase(myhash)
    myhash.keys.each do |key|
      value = myhash.delete(key)
      key = result_map[key] || (key.to_s.downcase rescue key)
      myhash[key] = value
    end
    myhash
  end
end
