Puppet::Type.newtype(:iscsi_target) do
  ensurable do
    newvalue(:connected) do
      provider.connect
    end

    newvalue(:disconnected) do
      provider.disconnect
    end

    def retrieve
      if provider.respond_to?(:exists?)
        provider.exists? ? :connected : :disconnected
      else
        raise Puppet::DevError, "No ability to determine if #{@resource.class.name} exi    sts"
      end
    end
  end

  # TODO: may need composite namevar.
  newparam(:node_address, :namevar => true) do
    desc 'the iscsi target node address.'
  end

  newparam(:is_persistent) do
    desc 'is persistent enabled.'
    newvalues('$true', '$false')
  end

  newparam(:is_multipath_enabled) do
    desc 'is multipath enabled.'
    newvalues('$true', '$false')
  end

  newparam(:authentication_type) do
    desc 'authentication for target login'
    newvalues(:None,:OneWayCHAP,:MutualCHAP)
    defaultto(:None)
  end

  newparam(:initiator_instance_name) do
    desc 'the initiator instance name.'
  end

  newparam(:chap_username) do
    desc 'username for mutual chap'
  end

  newparam(:chap_secret) do
    desc 'CHAP secret for authentication'
  end

  newparam(:target_portal_address) do
    desc 'ipaddress or dns name of target portal'
  end

  newparam(:target_portal_port_number) do
    desc 'target portal port number'
    newvalues(/^\d+$/)
    defaultto(3260)
  end

  newparam(:storage_match_pattern) do
    desc 'Storage volume matching pattern'
    defaultto('')
  end

end
