# TODO: extract property code to generic module
# We need some of the nice wrappers to puppet type
#
vmware_module = Puppet::Module.find('vmware_lib', Puppet[:environment].to_s)
require File.join vmware_module.path, 'lib/puppet/property/vmware'

Puppet::Type.newtype(:sc_logical_network_definition) do
  ensurable

  newparam(:name, :namevar => true) do
    desc 'SCVMM logical network definition'
  end

  newparam(:logical_network) do
    desc 'SCVMM logical network to associate with the definition.'
  end

  newproperty(:subnet_vlans, :parent => Puppet::Property::VMware_Array_Hash,
    :array_matching => :all, :sort => :true , :key => 'vlan') do

    validate do |value|
      raise(ArgumentError, 'Missing subnet/vlan values') unless value.is_a? ::Hash and value.keys.sort == ['subnet', 'vlan']
    end
   
    # TODO: temporary if we need to override insync? 
    def munge_vlan(value)
      result = []
      value.each do |subnet|
        subnet['vlan'] = subnet['vlan'].to_s
        result += [subnet]
      end
      result 
    end

    # Note: do not replace with munge do |value| 
    # This allows override of vmware_array_hash munge method
    def munge(value)
      value['vlan'] = value['vlan'].to_s if value.include? 'vlan'
      value
    end
  end

  newproperty(:host_groups, :parent => Puppet::Property::VMware_Array,
    :array_matching => :all, :sort => :true) do
    desc 'The host groups the logical network definition is available.'

    defaultto(['All Hosts'])

    validate do |value|
      raise(ArgumentError, 'Host Groups must be under "All Hosts"') unless value =~ /^All Hosts/
    end
  end

  newparam(:inclusive) do 
    newvalues(:true, :false)
    defaultto(:true)
  end

  newparam(:skip_deletion) do
    newvalues(:true, :false)
    defaultto(:false)
  end

  newparam(:async) do
    desc 'Process the request asynchronously (no longer confirm operation completed successfully).'
    newvalues(:true,:false)
    defaultto(:false)
  end

  autorequire(:sc_logical_network) do
    self[:logical_network]
  end
end

