Puppet::Type.newtype(:sc_virtual_network_adapter) do
  ensurable

  newparam(:name, :namevar => true) do
    desc 'SCVMM virtual network adapter'

    munge do |value|
      vm, resource[:slot_id] = value.split(':') if value.include? ':'
      resource[:vm] = vm unless resource[:vm]
      value
    end
  end

  newparam(:slot_id) do
    desc 'The slot the network adapter resides'
  end

  # TODO: This should be a property
  newparam(:description) do
    desc 'Virtual network adapter description'
  end

  newparam(:enable_mac_address_spoofing) do
    newvalues(:true, :false)

    munge do |value|
      case value
      when TrueClass, :true
        '$true'
      when FalseClass, :false
        '$false'
      end
    end
  end

  newparam(:enable_vm_network_optimization) do
    newvalues(:true, :false)

    munge do |value|
      case value
      when TrueClass, :true
        '$true'
      when FalseClass, :false
        '$false'
      end
    end
  end

  newproperty(:ipv4_address_type) do
    newvalues(:dynamic, :static)
  end

  newproperty(:ipv6_address_type) do
    newvalues(:dynamic, :static)
  end
  
  newproperty(:logical_network) do
  end
  
  newproperty(:virtual_network) do
  end

  newproperty(:mac_address) do
    munge do |value|
      value.gsub(':', '-') if value.include?(':')
    end
  end

  newproperty(:mac_address_type) do
    newvalues(:dynamic, :static)
  end

  newparam(:vlan_enabled) do
    newvalues(:true, :false)

    munge do |value|
      case value
      when TrueClass, :true
        '$true'
      when FalseClass, :false
        '$false'
      when 'true'
        '$true'
      when 'false'
        '$false'
      end
    end
  end

  newproperty(:vlan_id) do
    newvalues(/\d+/)
  end

  newparam(:no_connection) do
    newvalues(:true, :false)
  end

  newparam(:vm) do
  end

  autorequire(:sc_virtual_machine) do
    self[:vm]
  end

  newparam(:async) do
    desc 'Process the request asynchronously (no longer confirm operation completed successfully).'
    newvalues(:true,:false)
    defaultto(:false)
  end
end
