require 'spec_helper'
require 'puppet/provider/winrm'

describe Puppet::Provider::Winrm do
  before :each do
    @provider = Puppet::Provider::Winrm.new
  end

  context 'quote strings' do
    it 'should double quote by default' do
      @provider.quote('hi').should == '"hi"'
    end

    it 'should double quote' do
      @provider.quote('hi', :double).should == '"hi"'
    end

    it 'should single quote' do
      @provider.quote('hi', :single).should == "'hi'"
    end

    it 'should not quote' do
      @provider.quote('hi', :none).should == "hi"
    end

    it 'fail with invalid options' do
      expect{@provider.quote('hi', :bad)}.to raise_error(ArgumentError)
    end
  end

  context 'generate powershell command' do
    it 'should terminate command by default' do
      @provider.ps_cmd('').should == ';'
    end

    it 'should terminate command correctly' do
      @provider.ps_cmd('',{},:terminate => true).should == ';'
      @provider.ps_cmd('',{},:terminate => false).should == ''
    end

    it 'should assign result to variable' do
      @provider.ps_cmd('Get-Process',{},:variable => '$foo').
        should == '$foo = (Get-Process);'
    end

    it 'should process boolean options' do
      @provider.ps_cmd('', { :arg => true }).should == ' -arg;'
      @provider.ps_cmd('', { :arg => :true }).should == ' -arg;'
      @provider.ps_cmd('', { :arg => false }).should == ';'
      @provider.ps_cmd('', { :arg => :false }).should == ';'
    end

    it 'should process powershell boolean options' do
      @provider.ps_cmd('', { :arg => '$true' }).should == ' -arg:$true;'
      @provider.ps_cmd('', { :arg => '$false' }).should == ' -arg:$false;'
    end

    it 'should process array options' do
      @provider.ps_cmd('', { :arg => [] }).should == ' -arg ;'
      @provider.ps_cmd('', { :arg => [1] }).should == ' -arg 1;'
      @provider.ps_cmd('', { :arg => [1,2] }).should == ' -arg 1, 2;'
    end

    it 'should process string options' do
      @provider.ps_cmd('', { :arg => 'abc' }).should == ' -arg abc;'
    end
  end

  context 'credential management' do
    it 'should generate correct credentials' do
      @provider.credential('user','pass').should == "$encrypt = (ConvertTo-SecureString 'pass' -AsPlainText -Force);$credential = (New-Object -TypeName System.Management.Automation.PSCredential -ArgumentList \"user\", $encrypt);"
    end
  end

  context 'downcase hash keys' do
    it 'downcase keys' do
      @provider.hashkey_downcase({'Abc' => 1}).should == {'abc' => 1}
    end

    it 'convert symbol to string keys' do
      @provider.hashkey_downcase({:a => 1}).should == {'a' => 1}
    end
  end
end
